@@ -652,6 +652,36 @@ void extract_parenthesis(gchar *str, gchar op, gchar cl)
 	*destp = '\0';
 }
 
+void extract_parenthesis_with_escape(gchar *str, gchar op, gchar cl)
+{
+	register gchar *srcp, *destp;
+	gint in_brace;
+
+	srcp = destp = str;
+
+	while ((srcp = strchr(srcp, op))) {
+		if (destp > str)
+			*destp++ = ' ';
+		++srcp;
+		in_brace = 1;
+		while (*srcp) {
+			if (*srcp == op)
+				in_brace++;
+			else if (*srcp == cl)
+				in_brace--;
+
+			if (in_brace == 0)
+				break;
+
+			if (*srcp == '\\' && *(srcp + 1) != '\0')
+				++srcp;
+
+			*destp++ = *srcp++;
+		}
+	}
+	*destp = '\0';
+}
+
 void extract_parenthesis_with_skip_quote(gchar *str, gchar quote_chr,
 					 gchar op, gchar cl)
 {
@@ -713,6 +743,25 @@ void extract_quote(gchar *str, gchar quote_chr)
 	}
 }
 
+void extract_quote_with_escape(gchar *str, gchar quote_chr)
+{
+	register gchar *sp, *dp;
+
+	if ((sp = strchr(str, quote_chr))) {
+		dp = sp;
+		++sp;
+		while (*sp) {
+			if (*sp == quote_chr)
+				break;
+			else if (*sp == '\\' && *(sp + 1) != '\0')
+				++sp;
+
+			*dp++ = *sp++;
+		}
+		*dp = '\0';
+	}
+}
+
 void eliminate_address_comment(gchar *str)
 {
 	register gchar *srcp, *destp;
@@ -1795,6 +1844,26 @@ gchar *uriencode_for_filename(const gchar *filename)
 	return enc;
 }
 
+gchar *uriencode_for_mailto(const gchar *mailto)
+{
+	const gchar *p = mailto;
+	gchar *enc, *outp;
+
+	outp = enc = g_malloc(strlen(mailto) * 3 + 1);
+
+	for (p = mailto; *p != '\0'; p++) {
+		if (*p == '+') {
+			*outp++ = '%';
+			get_hex_str(outp, *p);
+			outp += 2;
+		} else
+			*outp++ = *p;
+	}
+
+	*outp = '\0';
+	return enc;
+}
+
 gint scan_mailto_url(const gchar *mailto, gchar **to, gchar **cc, gchar **bcc,
 		     gchar **subject, gchar **inreplyto, gchar **body)
 {
