@@ -37,6 +37,7 @@
 #include "addrbook.h"
 #include "addrindex.h"
 #include "xml.h"
+#include "utils.h"
 
 #ifndef DEV_STANDALONE
 #include "prefs.h"
@@ -1105,7 +1106,7 @@ gint addrindex_write_to( AddressIndex *addrIndex, const gchar *newFile ) {
 	fileSpec = g_strconcat( addrIndex->filePath, G_DIR_SEPARATOR_S, newFile, NULL );
 	addrIndex->retVal = MGU_OPEN_FILE;
 #ifdef DEV_STANDALONE
-	fp = fopen( fileSpec, "wb" );
+	fp = g_fopen( fileSpec, "wb" );
 	g_free( fileSpec );
 	if( fp ) {
 		fputs( "<?xml version=\"1.0\" ?>\n", fp );
