@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2006 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2007 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -91,6 +91,7 @@
 
 #ifdef G_OS_WIN32
 #  include <windows.h>
+#  include <pbt.h>
 #  include <fcntl.h>
 #endif
 
@@ -113,8 +114,12 @@ static struct RemoteCmd {
 	gboolean status_full;
 	GPtrArray *status_folders;
 	GPtrArray *status_full_folders;
+	gchar *open_msg;
 	gboolean configdir;
 	gboolean exit;
+#ifdef G_OS_WIN32
+	gushort ipcport;
+#endif
 } cmd;
 
 static void parse_cmd_opt		(int		 argc,
@@ -125,6 +130,9 @@ static void parse_gtkrc_files		(void);
 static void setup_rc_dir		(void);
 static void check_gpg			(void);
 static void set_log_handlers		(gboolean	 enable);
+#ifdef G_OS_WIN32
+static void process_win32_message	(void);
+#endif
 
 static gchar *get_socket_name		(void);
 static gint prohibit_duplicate_launch	(void);
@@ -138,6 +146,7 @@ static void migrate_old_config		(void);
 
 static void open_compose_new		(const gchar	*address,
 					 GPtrArray	*attach_files);
+static void open_message		(const gchar	*path);
 
 static void send_queue			(void);
 
@@ -287,6 +296,10 @@ int main(int argc, char *argv[])
 
 	set_log_handlers(TRUE);
 
+#ifdef G_OS_WIN32
+	process_win32_message();
+#endif
+
 	account_read_config_all();
 	account_set_menu();
 	main_window_reflect_prefs_all();
@@ -352,6 +365,42 @@ static void cleanup_console(void)
 #endif
 }
 
+#ifdef G_OS_WIN32
+static void read_ini_file(void)
+{
+	static gushort ipcport = REMOTE_CMD_PORT;
+	static gchar *confdir = NULL;
+
+	static PrefParam param[] = {
+		{"ipcport", "50215", &ipcport, P_USHORT},
+		{"configdir", NULL, &confdir, P_STRING},
+
+		{NULL, NULL, NULL, P_OTHER}
+	};
+
+	gchar *file;
+
+	file = g_strconcat(get_startup_dir(), G_DIR_SEPARATOR_S, "sylpheed.ini",
+			   NULL);
+	if (!is_file_exist(file)) {
+		g_free(file);
+		return;
+	}
+
+	prefs_read_config(param, "Sylpheed", file,
+			  conv_get_locale_charset_str());
+	g_free(file);
+
+	cmd.ipcport = ipcport;
+	if (confdir) {
+		set_rc_dir(confdir);
+		g_free(confdir);
+		confdir = NULL;
+		cmd.configdir = TRUE;
+	}
+}
+#endif
+
 static void parse_cmd_opt(int argc, char *argv[])
 {
 	gint i;
@@ -423,6 +472,14 @@ static void parse_cmd_opt(int argc, char *argv[])
 				i++;
 				p = argv[i + 1];
 			}
+		} else if (!strncmp(argv[i], "--open", 6)) {
+			const gchar *p = argv[i + 1];
+
+			if (p && *p != '\0' && *p != '-') {
+				cmd.open_msg = g_locale_to_utf8
+					(p, -1, NULL, NULL, NULL);
+				i++;
+			}
 		} else if (!strncmp(argv[i], "--configdir", 11)) {
 			const gchar *p = argv[i + 1];
 
@@ -444,6 +501,13 @@ static void parse_cmd_opt(int argc, char *argv[])
 				cmd.configdir = TRUE;
 				i++;
 			}
+#ifdef G_OS_WIN32
+		} else if (!strncmp(argv[i], "--ipcport", 9)) {
+			if (argv[i + 1]) {
+				cmd.ipcport = atoi(argv[i + 1]);
+				i++;
+			}
+#endif
 		} else if (!strncmp(argv[i], "--exit", 6)) {
 			cmd.exit = TRUE;
 		} else if (!strncmp(argv[i], "--help", 6)) {
@@ -462,7 +526,11 @@ static void parse_cmd_opt(int argc, char *argv[])
 			g_print("%s\n", _("  --status [folder]...   show the total number of messages"));
 			g_print("%s\n", _("  --status-full [folder]...\n"
 				"                         show the status of each folder"));
+			g_print("%s\n", _("  --open folderid/msgnum open message in new window"));
 			g_print("%s\n", _("  --configdir dirname    specify directory which stores configuration files"));
+#ifdef G_OS_WIN32
+			g_print("%s\n", _("  --ipcport portnum      specify port for IPC remote commands"));
+#endif
 			g_print("%s\n", _("  --exit                 exit Sylpheed"));
 			g_print("%s\n", _("  --debug                debug mode"));
 			g_print("%s\n", _("  --help                 display this help and exit"));
@@ -563,6 +631,10 @@ static void app_init(void)
 	/* ignore SIGPIPE signal for preventing sudden death of program */
 	signal(SIGPIPE, SIG_IGN);
 #endif
+
+#ifdef G_OS_WIN32
+	read_ini_file();
+#endif
 }
 
 static void parse_gtkrc_files(void)
@@ -891,6 +963,52 @@ static void set_log_handlers(gboolean enable)
 #endif
 }
 
+#ifdef G_OS_WIN32
+static LRESULT CALLBACK
+wndproc(HWND hwnd, UINT message, WPARAM wparam, LPARAM lparam)
+{
+	if (message == WM_POWERBROADCAST) {
+		debug_print("WM_POWERBROADCAST received: wparam = %d\n",
+			    wparam);
+		if (wparam == PBT_APMSUSPEND || wparam == PBT_APMSTANDBY) {
+			debug_print("suspend now\n");
+			inc_autocheck_timer_remove();
+		} else if (wparam == PBT_APMRESUMESUSPEND ||
+			   wparam == PBT_APMRESUMESTANDBY) {
+			debug_print("resume now\n");
+			inc_autocheck_timer_set();
+		}
+	}
+
+	return DefWindowProc(hwnd, message, wparam, lparam);
+}
+
+static void process_win32_message(void)
+{
+	WNDCLASS wclass;
+	static HWND hwnd = NULL;
+	ATOM klass;
+	HINSTANCE hmodule = GetModuleHandle(NULL);
+
+	if (hwnd)
+		return;
+
+	memset(&wclass, 0, sizeof(WNDCLASS));
+	wclass.lpszClassName = "sylpheed-observer";
+	wclass.lpfnWndProc   = wndproc;
+	wclass.hInstance     = hmodule;
+
+	klass = RegisterClass(&wclass);
+	if (!klass)
+		return;
+
+	hwnd = CreateWindow(MAKEINTRESOURCE(klass), NULL, WS_POPUP,
+			    0, 0, 1, 1, NULL, NULL, hmodule, NULL);
+	if (!hwnd)
+		UnregisterClass(MAKEINTRESOURCE(klass), hmodule);
+}
+#endif
+
 static gchar *get_socket_name(void)
 {
 	static gchar *filename = NULL;
@@ -921,13 +1039,13 @@ static gint prohibit_duplicate_launch(void)
 		return -1;
 	}
 	if (GetLastError() != ERROR_ALREADY_EXISTS) {
-		sock = fd_open_inet(REMOTE_CMD_PORT);
+		sock = fd_open_inet(cmd.ipcport ? cmd.ipcport : REMOTE_CMD_PORT);
 		if (sock < 0)
 			return 0;
 		return sock;
 	}
 
-	sock = fd_connect_inet(REMOTE_CMD_PORT);
+	sock = fd_connect_inet(cmd.ipcport ? cmd.ipcport : REMOTE_CMD_PORT);
 	if (sock < 0)
 		return -1;
 #else
@@ -1005,6 +1123,12 @@ static gint prohibit_duplicate_launch(void)
 			if (!strncmp(buf, ".\n", 2)) break;
 			fputs(buf, stdout);
 		}
+	} else if (cmd.open_msg) {
+		gchar *str;
+
+		str = g_strdup_printf("open %s\n", cmd.open_msg);
+		fd_write_all(sock, str, strlen(str));
+		g_free(str);
 	} else if (cmd.exit) {
 		fd_write_all(sock, "exit\n", 5);
 	} else {
@@ -1130,6 +1254,13 @@ static gboolean lock_socket_input_cb(GIOChannel *source, GIOCondition condition,
 		fd_write_all(sock, ".\n", 2);
 		g_free(status);
 		if (folders) g_ptr_array_free(folders, TRUE);
+	} else if (!strncmp(buf, "open", 4)) {
+		strretchomp(buf);
+		if (strlen(buf) < 6 || buf[4] != ' ') {
+			fd_close(sock);
+			return TRUE;
+		}
+		open_message(buf + 5);
 	} else if (!strncmp(buf, "exit", 4)) {
 		fd_close(sock);
 		app_will_exit(TRUE);
@@ -1167,6 +1298,9 @@ static void remote_command_exec(void)
 
 		if (cmd.send)
 			send_queue();
+
+		if (cmd.open_msg)
+			open_message(cmd.open_msg);
 	}
 
 	if (cmd.attach_files) {
@@ -1182,6 +1316,10 @@ static void remote_command_exec(void)
 		g_ptr_array_free(cmd.status_full_folders, TRUE);
 		cmd.status_full_folders = NULL;
 	}
+	if (cmd.open_msg) {
+		g_free(cmd.open_msg);
+		cmd.open_msg = NULL;
+	}
 	if (cmd.exit) {
 		app_will_exit(TRUE);
 	}
@@ -1307,6 +1445,38 @@ static void open_compose_new(const gchar *address, GPtrArray *attach_files)
 	g_free(utf8addr);
 }
 
+static void open_message(const gchar *path)
+{
+	gchar *id;
+	gchar *msg;
+	gint num;
+	FolderItem *item;
+	MsgInfo *msginfo;
+	MessageView *msgview;
+
+	if (gtkut_window_modal_exist())
+		return;
+
+	id = g_path_get_dirname(path);
+	msg = g_path_get_basename(path);
+	num = to_number(msg);
+	item = folder_find_item_from_identifier(id);
+	debug_print("open folder id: %s (msg %d)\n", id, num);
+
+	if (num > 0 && item) {
+		msginfo = folder_item_get_msginfo(item, num);
+		if (msginfo) {
+			msgview = messageview_create_with_new_window();
+			messageview_show(msgview, msginfo, FALSE);
+			procmsg_msginfo_free(msginfo);
+		} else
+			debug_print("message %d not found\n", num);
+	}
+
+	g_free(msg);
+	g_free(id);
+}
+
 static void send_queue(void)
 {
 	GList *list;
